@@ -1,6 +1,6 @@
 __title__ = 'Calliope'
-__author__ = 'Stefan Pfenninger'
-__copyright__ = 'Copyright 2013-2017 Stefan Pfenninger'
+__author__ = 'Stefan Pfenninger and Calliope contributors listed in AUTHORS'
+__copyright__ = 'Copyright 2013-2017 Calliope contributors listed in AUTHORS'
 
 from ._version import __version__
 
@@ -9,3 +9,4 @@ from .parallel import Parallelizer
 from . import utils
 from . import read
 from . import analysis
+from . import examples
