@@ -1,5 +1,5 @@
 """
-Copyright (C) 2013-2017 Stefan Pfenninger.
+Copyright (C) 2013-2017 Calliope contributors listed in AUTHORS.
 Licensed under the Apache 2.0 License (see LICENSE file).
 
 cli.py
@@ -13,6 +13,7 @@ import contextlib
 import datetime
 import logging
 import os
+import pstats
 import shutil
 import sys
 import traceback
@@ -20,6 +21,7 @@ import traceback
 import click
 
 from . import core
+from . import examples
 from . import _version
 from .parallel import Parallelizer
 
@@ -70,7 +72,8 @@ def format_exceptions(debug=False, pdb=False, profile=False, profile_filename=No
                 profile.dump_stats(dump_path)
             else:
                 print('\n\n----PROFILE OUTPUT----\n\n')
-                profile.print_stats()
+                stats = pstats.Stats(profile).sort_stats('cumulative')
+                stats.print_stats(20)  # Print first 20 lines
 
     except Exception as e:
         if debug:
@@ -119,19 +122,21 @@ def cli(ctx, version):
 
 @cli.command(short_help='create model')
 @click.argument('path')
+@click.option('--template', type=str, default=None)
 @_debug
-def new(path, debug):
+def new(path, template, debug):
     """
-    Create new model at the given PATH, based on the included example model.
-    The directory must not yet exist, and intermediate directories will
-    be created automatically.
+    Create new model at the given ``path``, based on one of the built-in
+    example models. The target path must not yet exist. Intermediate
+    directories will be created automatically.
     """
     if debug:
         print(_get_version())
-    # Copies the included example model
-    example_model = os.path.join(os.path.dirname(__file__), 'example_model')
-    click.echo('Creating new model in: {}'.format(path))
-    shutil.copytree(example_model, path)
+    if template is None:
+        template = 'NationalScale'
+    source_path = examples.PATHS[template]
+    click.echo('Copying {} template to target directory: {}'.format(template, path))
+    shutil.copytree(source_path, path)
 
 
 @cli.command(short_help='directly run single model')
