@@ -9,6 +9,8 @@ Basic model constraints.
 
 """
 
+import warnings
+
 import pyomo.core as po  # pylint: disable=import-error
 import numpy as np
 import xarray as xr
@@ -17,6 +19,15 @@ from .. import exceptions
 from .. import transmission
 from .. import utils
 
+formatwarning_orig = warnings.formatwarning
+def _formatwarning(message, category, filename, lineno, line=None):
+    """Formats ModelWarnings as "Warning: message" without extra crud"""
+    if category == exceptions.ModelWarning:
+        return 'Warning: ' + str(message) + '\n'
+    else:
+        return formatwarning_orig(message, category, filename, lineno, line)
+
+warnings.formatwarning = _formatwarning
 
 def get_constraint_param(model, param_string, y, x, t):
     """
@@ -106,6 +117,11 @@ def generate_variables(model):
     m.cost_fixed = po.Var(m.y, m.x, m.k, within=po.Reals)
     m.cost = po.Var(m.y, m.x, m.k, within=po.Reals)
 
+    # Binary/Integer variables
+    m.purchased = po.Var(m.y_purchase, m.x_purchase, within=po.Binary)
+    m.units = po.Var(m.y_milp, m.x_milp, within=po.NonNegativeIntegers)
+    m.operating_units = po.Var(m.y_milp, m.x_milp, m.t, within=po.NonNegativeIntegers)
+
 
 def node_resource(model):
     m = model.m
@@ -121,6 +137,15 @@ def node_resource(model):
     def r_available_rule(m, y, x, t):
         r_scale = model.get_option(y + '.constraints.r_scale', x=x)
         force_r = get_constraint_param(model, 'force_r', y, x, t)
+        resource = get_constraint_param(model, 'r', y, x, t)
+        if resource == np.inf and force_r is True:
+            w = exceptions.ModelWarning
+            message = ('`force_r: True` and `r: inf` are incompatible for {}:{}'
+                ' at {}. r_available constraint has been ignored. To avoid this'
+                ' message in future, set timeseries `r` at technology definition,'
+                ' even if you use location override.'.format(y, x, t))
+            warnings.warn(message, w)
+            return po.Constraint.Skip
 
         if y in m.y_sd:
             e_eff = get_constraint_param(model, 'e_eff', y, x, t)
@@ -131,10 +156,9 @@ def node_resource(model):
             c_con = sum(m.c_con[c, y, x, t] for c in m.c) * e_eff # demand techs
 
             if y in m.y_sd_r_area:
-                r_avail = (get_constraint_param(model, 'r', y, x, t) * r_scale
-                           * m.r_area[y, x])
+                r_avail = resource * r_scale * m.r_area[y, x]
             else:
-                r_avail = (get_constraint_param(model, 'r', y, x, t) * r_scale)
+                r_avail = get_constraint_param(model, 'r', y, x, t) * r_scale
 
             if force_r:
                 return c_prod + c_con == r_avail
@@ -145,10 +169,9 @@ def node_resource(model):
             r_eff = get_constraint_param(model, 'r_eff', y, x, t)
 
             if y in m.y_sp_r_area:
-                r_avail = (get_constraint_param(model, 'r', y, x, t) * r_scale
-                           * m.r_area[y, x] * r_eff)
+                r_avail = resource * r_scale * m.r_area[y, x] * r_eff
             else:
-                r_avail = get_constraint_param(model, 'r', y, x, t) * r_scale * r_eff
+                r_avail = resource * r_scale * r_eff
 
             if force_r:
                 return m.r[y, x, t] == r_avail
@@ -159,33 +182,35 @@ def node_resource(model):
     m.c_r_available = po.Constraint(m.y_finite_r, m.x_r, m.t,
                                     rule=r_available_rule)
 
+def unit_commitment(model):
+    m = model.m
+
+    def c_unit_commitment_rule(m, y, x, t):
+        # operating_units
+        # ^^^^^^^^^^^^^^^
+        #
+        # Constraining the number of integer units
+        # :math:`operating_units(y, x, t)` of a technology which
+        # can operate in a given timestep, based on maximum purchased units
+        # :math:`units(y, x)`
+        #
+        # .. math::
+        #
+        #   $operating\_units(y, x, t) \leq units(y, x)
+        ##
+
+        return m.operating_units[y, x, t] <= m.units[y, x]
+
+    m.c_unit_commitment = po.Constraint(m.y_milp, m.x_milp, m.t,
+                                        rule=c_unit_commitment_rule)
 
 def node_energy_balance(model):
     m = model.m
     time_res = model.data['_time_res'].to_series()
-
-    def get_e_eff_per_distance(model, y, x):
-        e_loss = model.get_option(y + '.constraints_per_distance.e_loss', x=x)
-        per_distance = model.get_option(y + '.per_distance')
-        tech, x2 = y.split(':')
-        link = model.config_model.get_key(
-            'links.' + x + ',' + x2,
-            default=model.config_model['links'].get(x2 + ',' + x)
-        )
-        # link = None if no link exists
-        if not link or tech not in link.keys():
-            return 1.0
-        try:
-            distance = link.get_key(tech + '.distance')
-        except KeyError:
-            if e_loss > 0:
-                e = exceptions.OptionNotSetError
-                raise e('Distance must be defined for link: {} '
-                        'and transmission tech: {}, as e_loss per distance '
-                        'is defined'.format(x + ',' + x2, tech))
-            else:
-                return 1.0
-        return 1 - (e_loss * (distance / per_distance))
+    e_eff_per_distance_getter = utils.e_eff_per_distance_getter(model.config_model)
+    @utils.memoize
+    def _e_eff_per_distance(y, x):
+        return e_eff_per_distance_getter(y, x)
 
     def get_conversion_out(c_1, c_2, m, y, x, t):
         if isinstance(c_1, dict):
@@ -223,7 +248,7 @@ def node_energy_balance(model):
             return (m.c_prod[c, y, x, t]
                     == -1 * m.c_con[c, y_remote, x_remote, t]
                     * e_eff
-                    * get_e_eff_per_distance(model, y, x))
+                    * _e_eff_per_distance(y, x))
         else:
             return po.Constraint.NoConstraint
 
@@ -350,7 +375,6 @@ def node_constraints_build(model):
     def get_var_constraint(model_var, y, var, x,
                            _equals=None, _max=None, _min=None,
                            scale=None):
-
         if not _equals:
             _equals = model.get_option(y + '.constraints.'
                                        + var + '.equals', x=x)
@@ -362,7 +386,7 @@ def node_constraints_build(model):
             _equals = scale * _equals
             _min = scale * _min
             _max = scale * _max
-        if _equals:
+        if _equals is not False and _equals is not None:
             if np.isinf(_equals):
                 e = exceptions.ModelError
                 raise e('Cannot use inf in operational mode, for value of '
@@ -394,7 +418,12 @@ def node_constraints_build(model):
         # incorporate timeseries resolution. Currently assumes that each
         # timestep is worth one unit of time.
         s_time_max = model.get_option(y + '.constraints.s_time.max', x=x)
-        s_cap_max = model.get_option(y + '.constraints.s_cap.max', x=x)
+        if y in m.y_milp and x in m.x_milp:
+            units = model.get_option(y + '.constraints.units.max', x=x)
+            s_cap_max = (units *
+                model.get_option(y + '.constraints.s_cap_per_unit', x=x))
+        else:
+            s_cap_max = model.get_option(y + '.constraints.s_cap.max', x=x)
         if not s_cap_max:
             s_cap_max = np.inf
         if not s_time_max:
@@ -436,9 +465,16 @@ def node_constraints_build(model):
         s_cap * charge rate = e_cap must hold. Otherwise, take the lowest capacity
         capacity defined by s_cap.max or e_cap.max / charge rate.
         """
-        s_cap_equals = model.get_option(y + '.constraints.s_cap.equals', x=x)
-        scale = model.get_option(y + '.constraints.e_cap_scale', x=x)
-        e_cap = model.get_option(y + '.constraints.e_cap.equals', x=x) * scale
+        if y in m.y_milp and x in m.x_milp:
+            units_equals = model.get_option(y + '.constraints.units.equals', x=x)
+            s_cap_equals = (units_equals *
+                model.get_option(y + '.constraints.s_cap_per_unit', x=x))
+            e_cap = (units_equals *
+                model.get_option(y + '.constraints.e_cap_per_unit', x=x))
+        else:
+            s_cap_equals = model.get_option(y + '.constraints.s_cap.equals', x=x)
+            scale = model.get_option(y + '.constraints.e_cap_scale', x=x)
+            e_cap = model.get_option(y + '.constraints.e_cap.equals', x=x) * scale
         charge_rate = model.get_option(y + '.constraints.c_rate', x=x)
         if e_cap and s_cap_equals and charge_rate and s_cap_equals * charge_rate != e_cap:
             raise exceptions.ModelError(
@@ -447,7 +483,11 @@ def node_constraints_build(model):
                 .format(y, x)
             )
         if not e_cap:
-            e_cap = model.get_option(y + '.constraints.e_cap.max', x=x) * scale
+            if y in m.y_milp and x in m.x_milp:
+                e_cap = (model.get_option(y + '.constraints.units.max', x=x)
+                    * model.get_option(y + '.constraints.e_cap_per_unit', x=x))
+            else:
+                e_cap = model.get_option(y + '.constraints.e_cap.max', x=x) * scale
         if not s_cap_equals:
             s_cap_max = get_s_cap(y, x, e_cap, charge_rate)
             return get_var_constraint(m.s_cap[y, x], y, 's_cap', x, _max=s_cap_max)
@@ -483,17 +523,96 @@ def node_constraints_build(model):
         # First check whether this tech is allowed at this location
         if not model._locations.at[x, y] == 1:
             return m.e_cap[y, x] == 0
+
+        # Addition of binary variable describing whether a technology has been
+        # purchased or not
+        if y in m.y_purchase and x in m.x_purchase:
+            purchased = m.purchased[y, x]
+        else:
+            purchased = 1
+
+        # Addition of integer variable describing how many units of a technology
+        # have been purchased
+        if y in m.y_milp and x in m.x_milp:
+            return m.e_cap[y, x] == (m.units[y, x]
+                    * model.get_option(y + '.constraints.e_cap_per_unit', x=x))
+
+        # parameters for transmission technologies are stored in a different
+        # location to other technologies
+        e_cap_max = model.get_option(y + '.constraints.e_cap.max', x=x)
+        e_cap_equals = model.get_option(y + '.constraints.e_cap.equals', x=x)
+        if y in m.y_transmission:
+            e_cap_equals_override = model._locations.get(
+                '_override.{}.constraints.e_cap.equals'.format(y), {x: None})[x]
+            e_cap_max_override = model._locations.get(
+                '_override.{}.constraints.e_cap.max'.format(y), {x: None})[x]
+            if e_cap_equals_override is not None and not np.isnan(e_cap_equals_override):
+                e_cap_equals = e_cap_equals_override
+            if  e_cap_max_override is not None and not np.isnan(e_cap_max_override):
+                e_cap_max = e_cap_max_override
+
         e_cap_scale = model.get_option(y + '.constraints.e_cap_scale', x=x)
+
+        # e_cap.equals forces an equality constraint, which can't be infinite
+        if e_cap_equals:
+            if e_cap_equals is None or np.isinf(e_cap_equals) or np.isnan(e_cap_equals):
+                e = exceptions.ModelError
+                raise e('Cannot use inf, NaN, or None for equality constraint: '
+                        '{}.e_cap.equals.{}'.format(y, x))
+            else:
+                return m.e_cap[y, x] == e_cap_equals * e_cap_scale * purchased
+
+        # In operation mode, e_cap is forced to an equality constraint, even if
+        # e_cap.max is defined.
+        if model.mode == 'operate' and y not in m.y_demand:
+            if e_cap_max is None or np.isinf(e_cap_max) or np.isnan(e_cap_max):
+                e = exceptions.ModelError
+                raise e('Cannot use inf, NaN, or None in operational mode, '
+                        'for value of {}.e_cap.max.{}'.format(y, x))
+            return m.e_cap[y, x] == e_cap_max * e_cap_scale * purchased
+
+        # Infinite or undefined e_cap.max leads to an ignored constraint
+        elif e_cap_max is None or np.isnan(e_cap_max) or np.isinf(e_cap_max):
+            return po.Constraint.Skip
+        else:
+            return m.e_cap[y, x] <= e_cap_max * e_cap_scale * purchased
+
+    def c_e_cap_min_rule(m, y, x):
+        """
+        Set minimum e_cap. All technologies.
+        """
+        # First check whether this tech is allowed at this location
+        if not model._locations.at[x, y] == 1:
+            return po.Constraint.Skip
+
+        # Addition of binary variable describing whether a technology has been
+        # purchased or not
+        if y in m.y_purchase and x in m.x_purchase:
+            purchased = m.purchased[y, x]
+        else:
+            purchased = 1
+
+        # Addition of integer variable describing how many units of a technology
+        # have been purchased
+        if y in m.y_milp and x in m.x_milp:
+            return po.Constraint.Skip
+
+        # parameters for transmission technologies are stored in a different
+        # location to other technologies
         if y in m.y_transmission:
             e_cap_equals = model._locations.get(
                 '_override.{}.constraints.e_cap.equals'.format(y), {x: None})[x]
-            e_cap_max = model._locations.get(
-                '_override.{}.constraints.e_cap.max'.format(y), {x: None})[x]
+            e_cap_min = model._locations.get(
+                '_override.{}.constraints.e_cap.min'.format(y), {x: None})[x]
         else:
-            e_cap_max = None
-            e_cap_equals = None
-        return get_var_constraint(m.e_cap[y, x], y, 'e_cap', x, scale=e_cap_scale,
-                                  _max=e_cap_max, _equals=e_cap_equals)
+            e_cap_min = model.get_option(y + '.constraints.e_cap.min', x=x)
+            e_cap_equals = model.get_option(y + '.constraints.e_cap.equals', x=x)
+        if e_cap_equals or model.mode == 'operate' or not e_cap_min:
+            return po.Constraint.Skip
+        else:
+            e_cap_scale = model.get_option(y + '.constraints.e_cap_scale', x=x)
+            return m.e_cap[y, x] >= e_cap_min * e_cap_scale * purchased
+
 
     def c_e_cap_storage_rule(m, y, x):
         """
@@ -533,14 +652,32 @@ def node_constraints_build(model):
         else:  # If ``r2_cap_follow`` not set, set up standard constraints
             return get_var_constraint(m.r2_cap[y, x], y, 'r2_cap', x)
 
+    def c_units_rule(m, y, x):
+        # units
+        # ^^^^^^^^
+        #
+        # Constraining the number of integer purchased units of a technology
+        # :math:`units(y, x)` to be between the user-defined maximum
+        # :math:`units.max(y, x)` and minimum :math:`units.min(y, x)`, or
+        # equal to :math:`units.equals(y, x)`.
+        #
+        # .. math::
+        #
+        #   $if units.equals(y, x) then units(y, x) = units.equals(y, x) else
+        #   $units.min(y, x) \leq units(y, x) \leq units.min(y, x)
+        ##
+
+        return get_var_constraint(m.units[y, x], y, 'units', x)
+
     # Constraints
     m.c_s_cap = po.Constraint(m.y_store, m.x_store, rule=c_s_cap_rule)
     m.c_r_cap = po.Constraint(m.y_sp_finite_r, m.x_r, rule=c_r_cap_rule)
     m.c_r_area = po.Constraint(m.y_r_area, m.x_r, rule=c_r_area_rule)
     m.c_e_cap = po.Constraint(m.y, m.x, rule=c_e_cap_rule)
+    m.c_e_cap_min = po.Constraint(m.y, m.x, rule=c_e_cap_min_rule)
     m.c_e_cap_storage = po.Constraint(m.y_store, m.x_store, rule=c_e_cap_storage_rule)
     m.c_r2_cap = po.Constraint(m.y_sp_r2, m.x_r, rule=c_r2_cap_rule)
-
+    m.c_units = po.Constraint(m.y_milp, m.x_milp, rule=c_units_rule)
 
 def node_constraints_operational(model):
     m = model.m
@@ -561,9 +698,10 @@ def node_constraints_operational(model):
         c_prod = m.c_prod[c, y, x, t]
         if y in m.y_conversion_plus:  # Conversion techs with 2 or more output carriers
             carriers_out = model.get_carrier(y, 'out', all_carriers=True)
-            if c not in carriers_out:
-                return c_prod == 0
-            if c in carriers_out and model._locations.at[x, y] == 0:
+            if isinstance(carriers_out, str):
+                carriers_out = tuple([carriers_out])
+            if (c not in carriers_out) or (c in carriers_out and
+                                           model._locations.at[x, y] == 0):
                 return c_prod == 0
             else:
                 return po.Constraint.Skip
@@ -571,6 +709,10 @@ def node_constraints_operational(model):
             return c_prod == 0
         p_eff = model.get_option(y + '.constraints.p_eff', x=x)
         if c == model.get_option(y + '.carrier', default=y + '.carrier_out'):
+            if y in m.y_milp and x in m.x_milp:
+                e_cap = model.get_option(y + '.constraints.e_cap_per_unit', x=x)
+                return c_prod <= (time_res.at[t] * m.operating_units[y, x, t]
+                                  * e_cap * p_eff)
             return c_prod <= time_res.at[t] * m.e_cap[y, x] * p_eff
         else:
             return m.c_prod[c, y, x, t] == 0
@@ -586,6 +728,10 @@ def node_constraints_operational(model):
         if y in m.y_conversion_plus:  # Conversion techs with 2 output carriers
             return po.Constraint.Skip
         elif c == model.get_option(y + '.carrier', default=y + '.carrier_out'):
+            if y in m.y_milp and x in m.x_milp:
+                e_cap = model.get_option(y + '.constraints.e_cap_per_unit', x=x)
+                return (m.c_prod[c, y, x, t] >= time_res.at[t] *
+                    m.operating_units[y, x, t] * e_cap * min_use)
             return (m.c_prod[c, y, x, t]
                     >= time_res.at[t] * m.e_cap[y, x] * min_use)
         else:
@@ -605,6 +751,10 @@ def node_constraints_operational(model):
         if not allow_c_prod:
             return c_prod == 0
         else:
+            if y in m.y_milp and x in m.x_milp:
+                e_cap = model.get_option(y + '.constraints.e_cap_per_unit', x=x)
+                return c_prod <= (time_res.at[t] * m.operating_units[y, x, t]
+                                  * e_cap)
             return c_prod <= time_res.at[t] * m.e_cap[y, x]
 
     def c_conversion_plus_prod_min_rule(m, y, x, t):
@@ -622,6 +772,10 @@ def node_constraints_operational(model):
                 c_prod = sum(m.c_prod[c, y, x, t] for c in c_out.keys())
             else:
                 c_prod = m.c_prod[c_out, y, x, t]
+            if y in m.y_milp and x in m.x_milp:
+                e_cap = model.get_option(y + '.constraints.e_cap_per_unit', x=x)
+                return c_prod >= (time_res.at[t] * m.operating_units[y, x, t]
+                                  * e_cap * min_use)
             return c_prod >= c_prod_min
 
     def c_con_max_rule(m, c, y, x, t):
@@ -638,6 +792,10 @@ def node_constraints_operational(model):
                 return po.Constraint.Skip
         if (allow_c_con is True and
                 c == model.get_option(y + '.carrier', default=y + '.carrier_in')):
+            if y in m.y_milp and x in m.x_milp:
+                e_cap = model.get_option(y + '.constraints.e_cap_per_unit', x=x)
+                return m.c_con[c, y, x, t] >= (time_res.at[t] * e_cap * p_eff
+                                               * m.operating_units[y, x, t] * -1)
             return m.c_con[c, y, x, t] >= (-1 * time_res.at[t]
                                             * m.e_cap[y, x] * p_eff)
         else:
@@ -663,10 +821,14 @@ def node_constraints_operational(model):
         """
         Set maximum export. All exporting technologies.
         """
+        if y in m.y_milp and x in m.x_milp:
+            operating_units = m.operating_units[y, x, t]
+        else:
+            operating_units = 1
         if get_constraint_param(model, 'export_cap', y, x, t):
             return (m.export[y, x, t] <=
-                    get_constraint_param(model, 'export_cap', y, x, t))
-
+                    get_constraint_param(model, 'export_cap', y, x, t)
+                    * operating_units)
         else:
             return po.Constraint.Skip
 
@@ -709,7 +871,6 @@ def node_constraints_transmission(model):
     m.c_transmission_capacity = po.Constraint(m.y_transmission, m.x_transmission,
                                               rule=c_trans_rule)
 
-
 def node_costs(model):
 
     m = model.m
@@ -721,8 +882,8 @@ def node_costs(model):
     cost_per_distance_getter = utils.cost_per_distance_getter(model.config_model)
 
     @utils.memoize
-    def _depreciation_rate(y, k):
-        return depreciation_getter(y, k)
+    def _depreciation_rate(y, x, k):
+        return depreciation_getter(y, x, k)
 
     @utils.memoize
     def _cost(cost, y, k, x=None):
@@ -742,15 +903,16 @@ def node_costs(model):
         if y in m.y_transmission:
             # Divided by 2 for transmission techs because construction costs
             # are counted at both ends
-            unit_cost = (_cost(cost, y, k, x) + _cost_per_distance(cost, y, k, x)) / 2
+            cost_parameter = (_cost(cost, y, k, x) +
+                                _cost_per_distance(cost, y, k, x)) / 2
         else:
-            unit_cost = _cost(cost, y, k, x)
+            cost_parameter = _cost(cost, y, k, x)
 
         # Search the Pyomo constraint for this technology and location, see if it's set
 
-        if (y, x) in getattr(m, 'c_' + cost).keys() or unit_cost >= 0:
-            return unit_cost * getattr(m, cost)[y, x]
-        elif unit_cost < 0:
+        if (y, x) in getattr(m, 'c_' + cost).keys() or cost_parameter >= 0:
+            return cost_parameter * getattr(m, cost)[y, x]
+        elif cost_parameter < 0:
             raise e(cost + '.max must be defined for {}:{} '
                     'as cost is negative'.format(y, x))
 
@@ -777,10 +939,25 @@ def node_costs(model):
         else:
             cost_r2_cap = 0
 
-        cost_con = (_depreciation_rate(y, k) *
+        if y in m.y_purchase and x in m.x_purchase:
+            if y in m.y_transmission:
+                cost_purchase = m.purchased[y, x] * (_cost('purchase', y, k, x)
+                                + _cost_per_distance('purchase', y, k, x)) / 2
+            else:
+                cost_purchase = _cost('purchase', y, k, x) * m.purchased[y, x]
+        elif y in m.y_milp and x in m.x_milp:
+            if y in m.y_transmission:
+                cost_purchase = m.units[y, x] * (_cost('purchase', y, k, x)
+                                + _cost_per_distance('purchase', y, k, x)) / 2
+            else:
+                cost_purchase = _cost('purchase', y, k, x) * m.units[y, x]
+        else:
+            cost_purchase = 0
+
+        cost_con = (_depreciation_rate(y, x, k) *
             (sum(time_res * weights) / 8760) *
             (cost_s_cap + cost_r_cap + cost_r_area + cost_r2_cap +
-             cost_e_cap)
+             cost_e_cap + cost_purchase)
         )
         if _cost('om_fixed', y, k, x) < 0 and (y, x) not in m.c_e_cap.keys():
             raise exceptions.OptionNotSetError(
