@@ -210,7 +210,7 @@ class Model(object):
             cr = config_run
             assert isinstance(cr, utils.AttrDict)
             # we have no filename so we just use current date/time
-            self.run_id = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
+            self.run_id = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S_%f")
             # Use current working directory as config_run path
             self.config_run_path = os.getcwd()
         self.config_run = cr
@@ -517,7 +517,7 @@ class Model(object):
             not top level, e.g. level=3 gives `carrier_out_3`
         primary: bool, optional, default = False
             give `primary carrier` for a given technology, which is a carrier in
-            `carrier_out` given ass `primary carrier` in the technology definition
+            `carrier_out` given as `primary carrier` in the technology definition
         all_carriers: bool, optional, default = False
             give all carriers for tech y and given direction. For conversion_plus
             technologies, this will give an array of carriers, if more than one
@@ -781,18 +781,8 @@ class Model(object):
             _x = [i for i in _x if i in self.config_run.subset_x]
         self._sets['x'] = _x
 
-        # y: techs
-        sets_y = sets.init_set_y(self, _x)
-        self._sets = {**self._sets, **sets_y}
-
-        # x subsets
-        sets_x = sets.init_set_x(self)
-        self._sets = {**self._sets, **sets_x}
-        # c: carriers
-        sets_c = sets.init_set_c(self)
-        self._sets['c'] = sets_c
-
         # k: cost classes
+        # Moved earlier as it's now used in sets.init_set_y
         classes_c = [
             list(self.config_model.techs[k].costs.keys())
             for k in self.config_model.techs
@@ -807,6 +797,17 @@ class Model(object):
         # Remove any duplicates by going from list to set and back
         self._sets['k'] = list(set(classes_c))
 
+        # y: techs
+        sets_y = sets.init_set_y(self, _x)
+        self._sets = {**self._sets, **sets_y}
+
+        # x subsets
+        sets_x = sets.init_set_x(self)
+        self._sets = {**self._sets, **sets_x}
+        # c: carriers
+        sets_c = sets.init_set_c(self)
+        self._sets['c'] = sets_c
+
         # Locations settings matrix and transmission technologies
         self._locations = locations.generate_location_matrix(
             self.config_model.locations, techs=self._sets['y']
@@ -980,10 +981,10 @@ class Model(object):
                 series = dataset['r'].loc[{'y': y, 'x': x}].to_pandas()
                 err_suffix = 'for tech: {}, at location: {}'.format(y, x)
                 if base_tech == 'demand':
-                    err = 'Demand resource must be <=0, ' + err_suffix
+                    err = 'Demand resource must be <=0 and not NaN, ' + err_suffix
                     assert (series <= 0).all(), err
                 elif base_tech == 'supply':
-                    err = 'Supply resource must be >=0, ' + err_suffix
+                    err = 'Supply resource must be >=0 and not NaN, ' + err_suffix
                     assert (series >= 0).all(), err
 
     def read_data(self):
@@ -1245,7 +1246,7 @@ class Model(object):
         # Storage locations
         m.x_store = po.Set(
             initialize=self._sets['x_store'], within=m.x, ordered=True)
-        # Deman locations
+        # Demand locations
         m.x_demand = po.Set(
             initialize=self._sets['x_demand'], within=m.x, ordered=True)
         # Conversion locations
@@ -1254,6 +1255,12 @@ class Model(object):
         # Export locations
         m.x_export = po.Set(
             initialize=self._sets['x_export'], within=m.x, ordered=True)
+        # Locations that are associated with MILP variables
+        m.x_milp = po.Set(
+            initialize=self._sets['x_milp'], within=m.x, ordered=True)
+        # Binary purchase locations
+        m.x_purchase = po.Set(
+            initialize=self._sets['x_purchase'], within=m.x, ordered=True)
         # Cost classes
         m.k = po.Set(initialize=self._sets['k'], ordered=True)
 
@@ -1325,6 +1332,10 @@ class Model(object):
             initialize=self._sets['y_cp_3in'], within=m.y, ordered=True)
         # Technologies that allow export
         m.y_export = po.Set(initialize=self._sets['y_export'], within=m.y, ordered=True)
+        # Technologies that are associated with MILP variables
+        m.y_milp = po.Set(initialize=self._sets['y_milp'], within=m.y, ordered=True)
+        # Technologies that have a binary purchase variable
+        m.y_purchase = po.Set(initialize=self._sets['y_purchase'], within=m.y, ordered=True)
 
         # Timeseries
         for param in self.config_model.timeseries_constraints:
@@ -1371,6 +1382,7 @@ class Model(object):
 
         # 1. Required
         constr = [constraints.base.node_resource,
+                  constraints.base.unit_commitment,
                   constraints.base.node_energy_balance,
                   constraints.base.node_constraints_build,
                   constraints.base.node_constraints_operational,
@@ -1396,6 +1408,7 @@ class Model(object):
     def _log_time(self):
         self.run_times["end"] = time.time()
         self.run_times["runtime"] = int(time.time() - self.run_times["start"])
+        self.solution.attrs['run_time'] = self.run_times["runtime"]
         logging.info('Runtime: ' + str(self.run_times["runtime"]) + ' secs')
 
     def run(self, iterative_warmstart=True):
@@ -1578,12 +1591,15 @@ class Model(object):
         # Add model and run configuration
         self.solution.attrs['config_run'] = self.config_run
         self.solution.attrs['config_model'] = self.config_model
+        self.solution.attrs['calliope_version'] = __version__
+        # run_time is added in self._log_time()
 
     def load_solution(self):
         sol = self.get_node_variables()
         sol = sol.merge(self.get_totals())
         sol = sol.merge(self.get_node_parameters())
         sol = sol.merge(self.get_costs().to_dataset(name='costs'))
+        sol = sol.merge(self.get_costs_per_timestep())
         self.solution = sol
         self.process_solution()
 
@@ -1632,7 +1648,7 @@ class Model(object):
         return c.fillna(0)
 
     def get_node_variables(self):
-        detail = ['s', 'r', 'r2', 'export']
+        detail = ['s', 'r', 'r2', 'export', 'operating_units']
         p = xr.Dataset()
         p['e'] = self.get_c_sum()
         for v in detail:
@@ -1654,6 +1670,21 @@ class Model(object):
 
         return self.get_var('e_cap') * p_eff
 
+    def get_purchased_units(self):
+        # Return either 'purchased' variable or 'units', depending on whether the
+        # technology is in 'y_purchased' or 'y_milp'. All other technologies
+        # return 1 if e_cap > 0
+        m = self.m
+        e_cap = self.get_var('e_cap')
+        all_units = (e_cap / e_cap).replace(np.nan, 0)
+        for v in ['units', 'purchased']:
+            try:
+                new_data = self.get_var(v)
+                all_units.ix[new_data.index, new_data.columns] = new_data
+            except exceptions.ModelError:
+                continue
+        return all_units
+
     def get_node_parameters(self):
         detail = ['e_cap', 's_cap', 'r_cap', 'r_area', 'r2_cap']
         result = xr.Dataset()
@@ -1663,6 +1694,7 @@ class Model(object):
             except exceptions.ModelError:
                 continue
         result['e_cap_net'] = self.get_e_cap_net()
+        result['purchased_units'] = self.get_purchased_units()
         return result
 
     def get_costs(self, t_subset=None):
@@ -1684,6 +1716,9 @@ class Model(object):
 
             return cost_fixed + cost_variable
 
+    def get_costs_per_timestep(self):
+        return self.get_var('cost_var').to_dataset(name='costs_variable')
+
     def get_totals(self, t_subset=None, apply_weights=True):
         """Get total produced and consumed per technology and location."""
         if t_subset is None:
@@ -2064,7 +2099,6 @@ class Model(object):
         ``self.config_run.output.path``
 
         """
-        sol = self.solution
         store_file = os.path.join(self.config_run.output.path, 'solution.nc')
         # Raise error if file exists already, to make sure we don't destroy
         # existing data
@@ -2080,16 +2114,18 @@ class Model(object):
             logging.warning(message)
             store_file = alt_file
 
+        # Shallow copy of dataset to allow metadata serialization
+        temp_solution = self.solution.copy(deep=False)
+
         # Metadata
         for k in ['config_model', 'config_run']:
             # Serialize config dicts to YAML strings
-            sol.attrs[k] = sol.attrs[k].to_yaml()
-        sol.attrs['run_time'] = self.run_times["runtime"]
-        sol.attrs['calliope_version'] = __version__
+            temp_solution.attrs[k] = temp_solution.attrs[k].to_yaml()
 
-        encoding = {k: {'zlib': True, 'complevel': 4} for k in self.solution.data_vars}
-        self.solution.to_netcdf(store_file, format='netCDF4', encoding=encoding)
-        self.solution.close()  # Force-close NetCDF file after writing
+        encoding = {k: {'zlib': True, 'complevel': 4} for k in temp_solution.data_vars}
+        temp_solution.to_netcdf(store_file, format='netCDF4', encoding=encoding)
+        temp_solution.close()  # Force-close NetCDF file after writing
+        del temp_solution  # Kill that shallow copy
 
         return store_file  # Return the path to the NetCDF file we used
 
@@ -2101,10 +2137,8 @@ class Model(object):
 
         # Metadata
         md = utils.AttrDict()
-        md['config_run'] = self.config_run
-        md['config_model'] = self.config_model
-        md['run_time'] = self.run_times["runtime"]
-        md['calliope_version'] = __version__
+        for k in ['config_model', 'config_run', 'run_time', 'calliope_version']:
+            md[k] = self.solution.attrs[k]
         md.to_yaml(os.path.join(self.config_run.output.path, 'metadata.yaml'))
 
         return self.config_run.output.path
