@@ -38,12 +38,19 @@ Technologies:
     * ``m.y_cp_3in``: tertiary carrier(s) in
 * ``m.y_transmission``: all transmission technologies
 * ``m.y_unmet``: dummy supply technologies to log
+* ``m.y_export``: all technologies allowing export of their carrier outside the system
+* ``m.y_purchase``: technology defining a 'purchase' cost, thus triggering an associated binary decision variable
+* ``m.y_milp``: technology defining a 'units' maximum, minimum, or equality, thus triggering an associated integer decision variable
 
 Locations:
 
-* ``m.x_trans``: all transmission locations
+* ``m.x_transmission``: all transmission locations
 * ``m.x_r``: all locations which act as system sources/sinks
+* ``m.x_conversion``: all locations in which there are conversion/conversion_plus technologies
 * ``m.x_store``: all locations in which storage is allowed
+* ``m.x_export``: locations allowing 'y_export' technologies to export outside the system
+* ``m.x_purchase``: locations associated with 'y_purchased' technologies
+* ``m.x_milp``: locations associated with 'y_milp' technologies
 
 Shared subsets
 ==============
@@ -68,9 +75,14 @@ Technologies:
     * ``m.y_conversion``
     * ``m.y_conversion_plus``
     * ``m.y_transmission``
+    * ``m.y_unmet``
+    * ``m.y_export``
+    * ``m.y_purchase``
+    * ``m.y_milp``
 * ``m.y_sd``: all basic supply & demand technologies, includes:
     * ``m.y_demand``
     * ``m.y_supply``
+    * ``m.y_unmet``
 * ``m.y_store``: all technologies that have storage capabilities, includes:
     * ``m.y_storage``
     * ``m.y_supply_plus``
@@ -78,9 +90,13 @@ Technologies:
 Locations:
 
 * ``m.x``: all locations, includes:
-    * ``m.x_trans``
+    * ``m.x_transmission``
     * ``m.x_r``
+    * ``m.x_conversion``
     * ``m.x_store``
+    * ``m.x_export``
+    * ``m.x_purchase``
+    * ``m.x_milp``
 
 ###PART TO INCLUDE IN DOCUMENTATION ENDS HERE###
 
@@ -205,6 +221,19 @@ def init_set_y(model, _x):
     # subset of technologies allowing export
     _y_export = [y for y in _y
                  if any([model.get_option(y + '.export', x=x) for x in _x])]
+
+    # subset of technologies associated with the model MILP functionality
+    _y_milp = [y for y in _y if
+               any([model.get_option(y + '.constraints.units.max', x=x) +
+                    model.get_option(y + '.constraints.units.equals', x=x) +
+                    model.get_option(y + '.constraints.units.min', x=x)
+                    for x in _x])
+                  ]
+    # subset of technologies associated with a binary purchase variable
+    _y_purchase = [y for y in _y if y not in _y_milp and
+                   any([model.get_cost('purchase', y, k, x=x)
+                        for x in _x for k in model._sets['k']])
+                  ]
     sets = {
         'y': _y,
         'y_demand': _y_demand,
@@ -228,6 +257,8 @@ def init_set_y(model, _x):
         'techs_transmission': transmission_techs,
         'x_transmission': _x_trans,
         'y_export': _y_export,
+        'y_purchase': _y_purchase,
+        'y_milp': _y_milp,
         'x_transmission_plus': _x
     }
 
@@ -258,13 +289,22 @@ def init_set_x(model):
     # All locations in which energy is exported
     _x_export = [x for x in model._sets['x'] if
                      set(model._sets['y_export']).intersection(locations[x].techs)]
+    # All locations with technologies associated with a MILP variable
+    _x_milp = [x for x in model._sets['x'] if
+                     set(model._sets['y_milp']).intersection(locations[x].techs)]
+    # All locations with technologies associated with a binary purchase variable
+    _x_purchase = [x for x in model._sets['x'] if
+                     set(model._sets['y_purchase']).intersection(locations[x].techs)]
+
 
     return {
         'x_r': _x_r,
         'x_store': _x_store,
         'x_demand': _x_demand,
         'x_conversion': _x_conversion,
-        'x_export': _x_export
+        'x_export': _x_export,
+        'x_purchase': _x_purchase,
+        'x_milp': _x_milp
     }
 
 def init_set_c(model):
@@ -328,13 +368,15 @@ def init_y_trans(model):
                 for y in tree[x]:
                     # Allow the tech
                     model._locations.at[x, y] = 1
-                    # Add constraints if needed. Speficying one way transmission
-                    # leads to e_prod in the direction opposite to the defined
-                    # link being set to False. E.g. if 'a,b' is one way for tech
-                    # 'y' then y:a.constraints.e_prod @ x=b -> False
+                    # Add constraints if needed. Specifying one way transmission
+                    # leads to e_con in the direction of the link being set to
+                    # False. E.g. if 'a,b' is one way for transmission tech 'y'
+                    # then y:a.constraints.e_con @ x=b -> False. Here, this stops
+                    # transmission from b to a (no carrier can be consumed by the
+                    # tranmission technology at b).
                     for c in tree[x][y].keys_nested():
                         if c == 'constraints.one_way':
-                            colname = '_override.' + y + '.constraints.e_prod'
+                            colname = '_override.' + y + '.constraints.e_con'
                         else:
                             colname = '_override.' + y + '.' + c
                         if colname not in model._locations.columns:
@@ -343,6 +385,6 @@ def init_y_trans(model):
                                 and '{},{}'.format(y.split(':')[1], x) in
                                 model.config_model.links.keys()):
                             # set the option in model._locations and model.config_model.locations
-                            model.set_option(y + '.constraints.e_prod', False, x=x)
+                            model.set_option(y + '.constraints.e_con', False, x=x)
                         else:
                             model._locations.at[x, colname] = tree[x][y].get_key(c)
